import { firestore } from 'firebase/app';
import { BaseObject, WithId } from './types';
import { Query } from './Query';
export declare class BaseRepository<T extends BaseObject> {
    collectionReference?: firestore.CollectionReference<T>;
    documentReference?: firestore.DocumentReference<T>;
    /**
     * @return {Promise<string>} id
     */
    create(data: T & {
        id?: string;
    }): Promise<string>;
    find(): Promise<WithId<T> | undefined>;
    all(arg?: {
        limit: number;
    }): Promise<WithId<T>[]>;
    update(data: Partial<T>): Promise<void>;
    destroy(): Promise<void>;
    limit(num: number): Query<T>;
    limitToLast(num: number): Query<T>;
    orderBy(field: string, direction?: firestore.OrderByDirection): Query<T>;
    where(field: string, operator: firestore.WhereFilterOp, value: string | number | Date | null): Query<T>;
    protected convertFromDbData(data: BaseObject): BaseObject;
    private buildQuery;
}
