"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseRepository = void 0;
const errors_1 = require("./errors");
const helper_1 = require("./helper");
const Query_1 = require("./Query");
class BaseRepository {
    /**
     * @return {Promise<string>} id
     */
    create(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.collectionReference == null)
                throw errors_1.ERRORS.NO_COLLECTION_REFERENCE;
            const { id } = data, addData = __rest(data, ["id"]);
            if (id) {
                yield this.collectionReference.doc(id).set(addData);
                return id;
            }
            const addedDocumentReference = yield this.collectionReference.add(addData);
            return addedDocumentReference.id;
        });
    }
    find() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.documentReference == null)
                throw errors_1.ERRORS.NO_DOCUMENT_REFERENCE('Did you mean? getAll');
            const snap = yield this.documentReference.get();
            const data = helper_1.mergeIdFromSnap(snap);
            if (data === undefined)
                return undefined;
            return this.convertFromDbData(data);
        });
    }
    all(arg) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.collectionReference == null)
                throw Error();
            const snapshot = yield (arg ? this.collectionReference.limit(arg.limit) : this.collectionReference).get();
            return snapshot.docs.flatMap((doc) => {
                const data = helper_1.mergeIdFromSnap(doc);
                if (!data)
                    return [];
                return this.convertFromDbData(data);
            });
        });
    }
    update(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.documentReference == null)
                throw errors_1.ERRORS.NO_DOCUMENT_REFERENCE();
            return this.documentReference.update(data);
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.documentReference == null)
                throw errors_1.ERRORS.NO_DOCUMENT_REFERENCE();
            return this.documentReference.delete();
        });
    }
    limit(num) {
        if (this.collectionReference == null)
            throw errors_1.ERRORS.NO_COLLECTION_REFERENCE();
        return this.buildQuery(this.collectionReference.limit(num));
    }
    limitToLast(num) {
        if (this.collectionReference == null)
            throw errors_1.ERRORS.NO_COLLECTION_REFERENCE();
        return this.buildQuery(this.collectionReference.limitToLast(num));
    }
    orderBy(field, direction) {
        if (this.collectionReference == null)
            throw errors_1.ERRORS.NO_COLLECTION_REFERENCE();
        return this.buildQuery(this.collectionReference.orderBy(field, direction));
    }
    where(field, operator, value) {
        if (this.collectionReference == null)
            throw errors_1.ERRORS.NO_COLLECTION_REFERENCE();
        return this.buildQuery(this.collectionReference.where(field, operator, value));
    }
    convertFromDbData(data) {
        return helper_1.convertTimestampToDate(data);
    }
    buildQuery(query) {
        return new Query_1.Query(query, this.convertFromDbData);
    }
}
exports.BaseRepository = BaseRepository;
